# Generated by Django 5.0 on 2024-02-08 12:33

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("contenttypes", "0002_remove_content_type_name"),
        ("flex_report", "0009_alter_template_buttons"),
    ]

    operations = [
        migrations.AlterField(
            model_name="column",
            name="model",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="flex_columns",
                to="contenttypes.contenttype",
            ),
        ),
        migrations.AlterField(
            model_name="template",
            name="model",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="flex_templates",
                to="contenttypes.contenttype",
                verbose_name="Model",
            ),
        ),
        migrations.AlterField(
            model_name="template",
            name="model_user_path",
            field=models.JSONField(
                blank=True,
                default={},
                max_length=200,
                null=True,
                verbose_name="Model User Path",
            ),
        ),
    ]
