# Generated by Django 5.0 on 2024-10-13 06:52

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("flex_report", "0016_column_column_type"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="column",
            name="groups",
        ),
        migrations.RemoveField(
            model_name="column",
            name="users",
        ),
        migrations.RemoveField(
            model_name="tablebutton",
            name="groups",
        ),
        migrations.RemoveField(
            model_name="tablebutton",
            name="users",
        ),
        migrations.RemoveField(
            model_name="tablepage",
            name="groups",
        ),
        migrations.RemoveField(
            model_name="tablepage",
            name="users",
        ),
        migrations.RemoveField(
            model_name="template",
            name="groups",
        ),
        migrations.RemoveField(
            model_name="template",
            name="users",
        ),
        migrations.AddField(
            model_name="tablebutton",
            name="is_table",
            field=models.BooleanField(default=False, verbose_name="Is Table"),
        ),
    ]
